// Program to display scan codes returned by keyboard controller.
//
// Previous versions attempted to place all of the scan codes
// associated with a key press-release on a separate line, but
// this feature never worked corretly for certain key, or key
// combinations, or keys pressed in rapid succession.
//
// To separate the scan codes for this version you must press
// the Return key to move to the next line.
//
// Press Escape to unhook the interrupt handler and terminate
// the program.

#include <stdlib.h>
#include <stdio.h>
#include <bios.h>
#include <dos.h>

void (_interrupt _far *prev_isr09)(void);
void _interrupt _far isr09(void);

unsigned char curr_scan, prev_scan;

int main( int argc, char *argv[], char *envp[] )
{
    prev_isr09 = _dos_getvect(9);
    _dos_setvect(9, isr09);
    for(;;)
    {
				if( curr_scan )
        {
						switch( curr_scan )
            {
                case 0x1c:
										if( prev_scan == 0xe0 || prev_scan == 0x1c )
												printf( "%.2X ", curr_scan );
                    else
												printf( "<%.2X ", curr_scan );
                    break;
                case 0x9c:
										if( prev_scan == 0xe0 )
												printf( "%.2X ", curr_scan );
                    else
												printf( "%.2X>\n", curr_scan );
                    break;
                case 0x01:
										if( prev_scan != 1 )
												printf( "\n" );
										printf( "%.2X ", curr_scan );
                    break;
                case 0x81:
										printf( "%.2X ", curr_scan );
										_dos_setvect( 9, prev_isr09 );
                    exit(0);
                default:
										printf( "%.2X ", curr_scan );
            }
            prev_scan = curr_scan;
            curr_scan = 0;
        }
				while( _bios_keybrd( _KEYBRD_READY ) )
						_bios_keybrd( _KEYBRD_READ );
    }
}

void _interrupt _far isr09( void )
{
    _asm
    {
				in  al,60h
				mov curr_scan,al
    }
    (*prev_isr09)();
}
